#include <string.h>

#include <dev/ppp.h>

#include <sys/heap.h>

#include <net/if_var.h>
#include <netinet/if_ppp.h>
#include <netinet/ppp_fsm.h>

#include <netinet/md5.h>

/*!
 * \addtogroup xgChap
 *
 */
/*@{*/

/*
 * No server support.
 */
void ChapRxAuthReq(NUTDEVICE *dev, u_char id, NETBUF *nb)
{
    PPPDCB *dcb = dev->dev_dcb;
    char *cp = nb->nb_ap.vp;
    u_short len = 0;
    u_char *Response = 0;
    u_char ChallengeSize = *cp;
    u_char Digest[16];
    md5_context ctx;

    /*
     * Extract the challenge chain and call to the md5 hash generator
     */
    if(dcb->dcb_pass)
        len = strlen(dcb->dcb_pass);
    if ((Response = NutHeapAlloc(1+len+ChallengeSize)) != 0) {
        memcpy(Response, &id, 1);
        memcpy(Response + 1, dcb->dcb_pass, len);
        memcpy(Response + 1+len, cp + 1, ChallengeSize);

        md5init( &ctx );
        md5update( &ctx, Response, 1+len+ChallengeSize );
        md5final( Digest, &ctx );

        NutHeapFree(Response);
    }

    /*
     * Create the request.
     */
    len = 0;
    if (dcb->dcb_user)
        len = strlen(dcb->dcb_user);
    /* Mhe:      should this 0 be replaced by nb ?????????????????????????????????????? */
    if ((nb = NutNetBufAlloc(0, NBAF_APPLICATION, 1+sizeof(Digest)+len)) != 0) {
        cp = nb->nb_ap.vp;
        *cp = sizeof(Digest);
        memcpy(cp + 1, Digest, sizeof(Digest));
        memcpy(cp + 1+sizeof(Digest), dcb->dcb_user, len);

        dcb->dcb_auth_state = CHAPCS_AUTHREQ;
        NutChapOutput(dev, CHAP_RESPONSE, id, nb);
    }
}

void ChapRxAuthAck(NUTDEVICE *dev, u_char id, NETBUF *nb)
{
    PPPDCB *dcb = dev->dev_dcb;

    if(dcb->dcb_auth_state == CHAPCS_AUTHREQ) {
        /*
         * Flag us open and start the network.
         */
        dcb->dcb_auth_state = CHAPCS_OPEN;
        
        // added
        if (dcb->dcb_ipcp_state != PPPS_OPENED) {
            IpcpLowerUp(dev);
        }
    }
}

void ChapRxAuthNak(NUTDEVICE *dev, u_char id, NETBUF *nb)
{
    PPPDCB *dcb = dev->dev_dcb;

    if(dcb->dcb_auth_state == CHAPCS_AUTHREQ) {
        dcb->dcb_auth_state = CHAPCS_BADAUTH;
        IpcpLowerDown(dev);
    }
}

/*!
 * \brief Handle incoming Chap packets.
 *
 *
 * \param dev Identifies the device that received the packet.
 * \param nb  Pointer to a network buffer structure containing 
 *            the Chap packet.
 */
void NutChapInput(NUTDEVICE * dev, NETBUF * nb)
{
    XCPHDR *chap;
    u_short len;

    /*
     * Drop packets with illegal lengths.
     */
    if (nb->nb_nw.sz < sizeof(XCPHDR)) {
        NutNetBufFree(nb);
        return;
    }
    chap = (XCPHDR *) nb->nb_nw.vp;
    if ((len = htons(chap->xch_len)) < sizeof(XCPHDR) || len > nb->nb_nw.sz) {
        NutNetBufFree(nb);
        return;
    }

    /*
     * Split the Chap packet.
     */
    nb->nb_ap.vp = chap + 1;
    nb->nb_ap.sz = htons(chap->xch_len) - sizeof(XCPHDR);

    /*
     * Action depends on code.
     */
    switch (chap->xch_code) {
    case CHAP_CHALLENGE:
        ChapRxAuthReq(dev, chap->xch_id, nb);
        break;

    case CHAP_SUCCESS:
        ChapRxAuthAck(dev, chap->xch_id, nb);
        break;

    case CHAP_FAILURE:
        ChapRxAuthNak(dev, chap->xch_id, nb);
        break;

    default:
        break;
    }
    NutNetBufFree(nb);
}

/*@}*/
