#include <string.h>

#include <dev/ppp.h>

#include <sys/heap.h>
#include <sys/thread.h>

#include <net/if_var.h>
#include <net/ppp.h>
#include <netinet/if_ppp.h>
#include <netinet/ppp_fsm.h>

/*!
 * \addtogroup xgChap
 */
/*@{*/

/*!
 * \brief Send a Chap packet.
 *
 * \note Applications typically do not call this function.
 *
 * \param dev   Identifies the device to use.
 * \param nb    Network buffer structure containing the packet to send
 *              or null if the packet contains no information.
 *              The structure must have been allocated by a previous
 *              call NutNetBufAlloc() and will be freed when this function
 *              returns.
 *
 * \return 0 on success, -1 in case of any errors.
 */
int NutChapOutput(NUTDEVICE * dev, u_char code, u_char id, NETBUF * nb)
{
    XCPHDR *xch;
        
    if ((nb = NutNetBufAlloc(nb, NBAF_NETWORK, sizeof(XCPHDR))) == 0)
        return -1;

    xch = nb->nb_nw.vp;
    xch->xch_code = code;
    xch->xch_id = id;
    xch->xch_len = htons(nb->nb_nw.sz + nb->nb_tp.sz + nb->nb_ap.sz);

    if (NutPppOutput(dev, PPP_CHAP, 0, nb) == 0)
        NutNetBufFree(nb);

    return 0;
}
